﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Runtime.InteropServices;

/// <summary>
///Dongle 的摘要说明
/// </summary>
public class Dongle
{
	public Dongle()
	{
		//
		//TODO: 在此处添加构造函数逻辑
		//
	}

    /************************************************************************/
    /*                                  定义                            */
    /************************************************************************/
    public static uint CONST_PID = 0xFFFFFFFF;  //出厂时默认的PID
    public static string CONST_USERPIN = "12345678";  //出厂时默认的USERPIN
    public static string CONST_ADMINPIN = "FFFFFFFFFFFFFFFF"; //出厂时默认的ADMINPIN

    public static uint PROTOCOL_HID = 0;//hid协议
    public static uint PROTOCOL_CCID = 1;//ccid协议

    //文件类型定义
    public static uint FILE_DATA = 1; //普通数据文件
    public static uint FILE_PRIKEY_RSA = 2;//RSA私钥文件
    public static uint FILE_PRIKEY_ECCSM2 = 3; //ECC或者SM2私钥文件(SM2私钥文件和ECC私钥文件结构相同，属相同文件类型)
    public static uint FILE_KEY = 4; //SM4和3DES密钥文件
    public static uint FILE_EXE = 5; //可执行文件

    //LED灯状态定义
    public static uint LED_OFF = 0; //灯灭
    public static uint LED_ON = 1; //灯亮
    public static uint LED_BLINK = 2; //灯闪

    //PIN码类型
    public static uint FLAG_USERPIN = 0; //用户PIN
    public static uint FLAG_ADMINPIN = 1; //开发商PIN

    //加解密标志
    public static uint FLAG_ENCODE = 0; //加密
    public static uint FLAG_DECODE = 1; //解密

    //HASH算法类型
    public static uint FLAG_HASH_MD5 = 0; //MD5     运算结果16字节
    public static uint FLAG_HASH_SHA1 = 1; //SHA1    运算结果20字节
    public static uint FLAG_HASH_SM3 = 2; //SM3     运算结果32字节

    //远程升级的功能号
    public static uint UPDATE_FUNC_CreateFile = 1; //创建文件
    public static uint UPDATE_FUNC_WriteFile = 2; //写文件
    public static uint UPDATE_FUNC_DeleteFile = 3; //删除文件
    public static uint UPDATE_FUNC_FileLic = 4; //设置文件授权
    public static uint UPDATE_FUNC_SeedCount = 5; //设置种子码可运算次数
    public static uint UPDATE_FUNC_DownloadExe = 6; //升级可执行文件
    public static uint UPDATE_FUNC_UnlockUserPin = 7; //解锁用户PIN
    public static uint UPDATE_FUNC_Deadline = 8; //时钟锁升级使用期限

    //x 锁的索引 y 模块号
    public static uint MAKEWORD(uint x, uint y)
    {
        return (((x << 16) & 0xFFFF0000) | ((y) & 0x0000FFFF));
    }

    /************************************************************************/
    /*                              结构                                    */
    /************************************************************************/
    //RSA公钥格式(兼容1024,2048)
    [StructLayout(LayoutKind.Sequential)]
    public struct RSA_PUBLIC_KEY
    {
        public uint bits;                   // length in bits of modulus        	
        public uint modulus;				  // modulus
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 256)]
        public byte[] exponent;       // public exponent
    }
    //RSA私钥格式(兼容1024,2048)
    [StructLayout(LayoutKind.Sequential)]
    public struct RSA_PRIVATE_KEY
    {
        public uint bits;                   // length in bits of modulus        	
        public uint modulus;				  // modulus  
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 256)]
        public byte[] publicExponent;       // public exponent
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 256)]
        public byte[] exponent;       // public exponent
    }
    //外部ECCSM2公钥格式 ECC(支持bits为192或256)和SM2的(bits为固定值0x8100)公钥格式
    [StructLayout(LayoutKind.Sequential)]
    public struct ECCSM2_PUBLIC_KEY
    {
        public uint bits;                   // length in bits of modulus        	
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
        public uint[] XCoordinate;       // 曲线上点的X坐标
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
        public uint[] YCoordinate;       // 曲线上点的Y坐标
    }
    //外部ECCSM2私钥格式 ECC(支持bits为192或256)和SM2的(bits为固定值0x8100)私钥格式  
    [StructLayout(LayoutKind.Sequential)]
    public struct ECCSM2_PRIVATE_KEY
    {
        public uint bits;                   // length in bits of modulus        	
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
        public uint[] PrivateKey;           // 私钥
    }

    //加密锁信息
    [StructLayout(LayoutKind.Sequential)]
    public struct DONGLE_INFO
    {
        public ushort m_Ver;               //COS版本,比如:0x0201,表示2.01版             	
        public ushort m_Type;              //产品类型: 0xFF表示标准版, 0x00为时钟锁,0x01为带时钟的U盘锁,0x02为标准U盘锁  
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
        public byte[] m_BirthDay;       //出厂日期 
        public uint m_Agent;             //代理商编号,比如:默认的0xFFFFFFFF
        public uint m_PID;               //产品ID
        public uint m_UserID;            //用户ID
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
        public byte[] m_HID;            //8字节的硬件ID
        public uint m_IsMother;          //母锁标志: 0x01表示是母锁, 0x00表示不是母锁
        public uint m_DevType;           //设备类型(PROTOCOL_HID或者PROTOCOL_CCID)
    }
    /*************************文件授权结构***********************************/
    //数据文件授权结构
    [StructLayout(LayoutKind.Sequential)]
    public struct DATA_LIC
    {
        public ushort m_Read_Priv;     //读权限: 0为最小匿名权限，1为最小用户权限，2为最小开发商权限            	
        public ushort m_Write_Priv;    //写权限: 0为最小匿名权限，1为最小用户权限，2为最小开发商权限
    }

    //私钥文件授权结构
    [StructLayout(LayoutKind.Sequential)]
    public struct PRIKEY_LIC
    {
        public uint m_Count;        //可调次数: 0xFFFFFFFF表示不限制, 递减到0表示已不可调用
        public byte m_Priv;         //调用权限: 0为最小匿名权限，1为最小用户权限，2为最小开发商权限
        public byte m_IsDecOnRAM;   //是否是在内存中递减: 1为在内存中递减，0为在FLASH中递减
        public byte m_IsReset;      //用户态调用后是否自动回到匿名态: TRUE为调后回到匿名态 (开发商态不受此限制)
        public byte m_Reserve;      //保留,用于4字节对齐
    }

    //对称加密算法(SM4/TDES)密钥文件授权结构
    [StructLayout(LayoutKind.Sequential)]
    public struct KEY_LIC
    {
        public uint m_Priv_Enc;   //加密时的调用权限: 0为最小匿名权限，1为最小用户权限，2为最小开发商权限
    }
    
    //可执行文件授权结构
    [StructLayout(LayoutKind.Sequential)]
    public struct EXE_LIC
    {
        public ushort m_Priv_Exe;   //运行的权限: 0为最小匿名权限，1为最小用户权限，2为最小开发商权限
    }

    /****************************文件属性结构********************************/
    //数据文件属性数据结构
    [StructLayout(LayoutKind.Sequential)]
    public struct DATA_FILE_ATTR
    {
        public uint m_Size;      //数据文件长度，该值最大为4096
        public DATA_LIC m_Lic;       //授权
    }

    //ECCSM2/RSA私钥文件属性数据结构
    [StructLayout(LayoutKind.Sequential)]
    public struct PRIKEY_FILE_ATTR
    {
        public ushort m_Type;       //数据类型:ECCSM2私钥 或 RSA私钥
        public ushort m_Size;       //数据长度:RSA该值为1024或2048, ECC该值为192或256, SM2该值为0x8100
        public PRIKEY_LIC m_Lic;        //授权
    }

    //对称加密算法(SM4/TDES)密钥文件属性数据结构
    [StructLayout(LayoutKind.Sequential)]
    public struct KEY_FILE_ATTR
    {
        public uint m_Size;       //密钥数据长度=16
        public KEY_LIC m_Lic;        //授权
    }

    //可执行文件属性数据结构
    [StructLayout(LayoutKind.Sequential)]
    public struct EXE_FILE_ATTR
    {
        public EXE_LIC m_Lic;        //授权	
        public ushort m_Len;        //文件长度
    }
    /*************************文件列表结构***********************************/
    //获取私钥文件列表时返回的数据结构
    [StructLayout(LayoutKind.Sequential)]
    public struct PRIKEY_FILE_LIST
    {
        public ushort m_FILEID;  //文件ID
        public ushort m_Reserve; //保留,用于4字节对齐
        public PRIKEY_FILE_ATTR m_attr;    //文件属性
    }

    //获取SM4及TDES密钥文件列表时返回的数据结构
    [StructLayout(LayoutKind.Sequential)]
    public struct KEY_FILE_LIST
    {
        public ushort m_FILEID;  //文件ID
        public ushort m_Reserve; //保留,用于4字节对齐
        public KEY_FILE_ATTR m_attr;    //文件属性
    }

    //获取数据文件列表时返回的数据结构
    [StructLayout(LayoutKind.Sequential)]
    public struct DATA_FILE_LIST
    {
        public ushort m_FILEID;  //文件ID
        public ushort m_Reserve; //保留,用于4字节对齐
        public DATA_FILE_ATTR m_attr;    //文件属性
    }

    //获取可执行文件列表时返回的数据结构
    [StructLayout(LayoutKind.Sequential)]
    public struct EXE_FILE_LIST
    {
        public ushort m_FILEID;    //文件ID
        public EXE_FILE_ATTR m_attr;
        public ushort m_Reserve;  //保留,用于4字节对齐
    }

    //下载和列可执行文件时填充的数据结构
    [StructLayout(LayoutKind.Sequential)]
    public struct EXE_FILE_INFO
    {
        public ushort m_dwSize;           //可执行文件大小
        public ushort m_wFileID;          //可执行文件ID
        public byte m_Priv;             //调用权限: 0为最小匿名权限，1为最小用户权限，2为最小开发商权限

        public byte[] m_pData;            //可执行文件数据
    }


    //需要发给空锁的初始化数据
    [StructLayout(LayoutKind.Sequential)]
    public struct SON_DATA
    {
        public uint m_SeedLen;                 //种子码长度
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 256)]
        public string m_SeedForPID;	       //产生产品ID和开发商密码的种子码 (最长250个字节)
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 18)]
        public string m_UserPIN;         //用户密码(16个字符的0终止字符串)
        public sbyte m_UserTryCount;            //用户密码允许的最大错误重试次数
        public sbyte m_AdminTryCount;           //开发商密码允许的最大错误重试次数
        //RSA_PRIVATE_KEY m_UpdatePriKey;   //远程升级私钥
        public uint m_UserID_Start;            //起始用户ID
    }

    //母锁数据
    [StructLayout(LayoutKind.Sequential)]
    public struct MOTHER_DATA
    {
        public SON_DATA m_Son;                  //子锁初始化数据
        public uint m_Count;                //可产生子锁初始化数据的次数 (-1表示不限制次数, 递减到0时会受限)
    }

    [DllImport("NetETARM_Dll.dll.dll")]
    static public extern uint Dongle_Enum(IntPtr pDongleInfo, out ushort pCount);//IntPtr为DONGLE_INFO结构

    [DllImport("NetETARM_Dll.dll.dll")]
    static public extern uint Dongle_Open(ref uint phDongle, uint nIndex);

    [DllImport("NetETARM_Dll.dll.dll")]
    static public extern uint Dongle_Close(uint hDongle);

    [DllImport("NetETARM_Dll.dll.dll")]
    static public extern uint Dongle_GenRandom(uint hDongle, uint nLen, byte[] pRandom);

    [DllImport("NetETARM_Dll.dll.dll")]
    static public extern uint Dongle_LEDControl(uint hDongle, uint nFlag);

    [DllImport("NetETARM_Dll.dll.dll")]
    static public extern uint Dongle_VerifyPIN(uint hDongle, uint nFlags, /*byte[]*/string pPIN, out uint pRemainCount);

    [DllImport("NetETARM_Dll.dll.dll")]
    static public extern uint Dongle_ReadFile(uint hDongle, short wFileID, short wOffset, byte[] buffer, uint nDataLen);

    [DllImport("NetETARM_Dll.dll.dll")]
    static public extern uint Dongle_RunExeFile(uint hDongle, short wFileID, byte[] pInOutData, short wInOutDataLen, ref uint nMainRet);

    [DllImport("NetETARM_Dll.dll.dll")]
    static public extern uint Dongle_WriteData(uint hDongle, uint nOffset, byte[] pData, uint nDataLen);

    [DllImport("NetETARM_Dll.dll.dll")]
    static public extern uint Dongle_ReadData(uint hDongle, uint nOffset, byte[] pData, uint nDataLen);

    [DllImport("NetETARM_Dll.dll.dll")]
    static public extern uint Dongle_GetUTCTime(uint hDongle, ref uint pdwUTCTime);

    [DllImport("NetETARM_Dll.dll.dll")]
    static public extern uint Dongle_SetDeadline(uint hDongle, uint dwTime);

    [DllImport("NetETARM_Dll.dll.dll")]
    static public extern uint Dongle_RsaPri(uint hDongle, ushort wPriFileID, uint nFlag, byte[] pInData, uint nInDataLen, byte[] pOutData, ref uint pOutDataLen);

    [DllImport("NetETARM_Dll.dll.dll")]
    static public extern uint Dongle_RsaPub(uint hDongle, uint nFlag, ref RSA_PUBLIC_KEY pPubKey, byte[] pInData, uint nInDataLen, byte[] pOutData, ref uint pOutDataLen);

    [DllImport("NetETARM_Dll.dll.dll")]
    static public extern uint Dongle_TDES(uint hDongle, ushort wKeyFileID, uint nFlag, byte[] pInData, byte[] pOutData, uint nDataLen);

    [DllImport("NetETARM_Dll.dll.dll")]
    static public extern uint Dongle_Seed(uint hDongle, byte[] pSeed, uint nSeedLen, byte[] pOutData);

    [DllImport("NetETARM_Dll.dll.dll")]
    static public extern uint Dongle_EccSign(uint hDongle, ushort wPriFileID, byte[] pHashData, uint nHashDataLen, byte[] pOutData);

    [DllImport("NetETARM_Dll.dll.dll")]
    static public extern uint Dongle_EccVerify(uint hDongle, ref ECCSM2_PUBLIC_KEY pPubKey, byte[] pHashData, uint nHashDataLen, byte[] pSign);

    
    //错误码
    public static uint DONGLE_SUCCESS = 0x00000000;             // 操作成功
    public static uint DONGLE_NOT_FOUND = 0xF0000001;           // 未找到指定的设备
    public static uint DONGLE_INVALID_HANDLE = 0xF0000002;	   // 无效的句柄
    public static uint DONGLE_INVALID_PARAMETER = 0xF0000003;   // 参数错误
    public static uint DONGLE_COMM_ERROR = 0xF0000004;		   // 通讯错误
    public static uint DONGLE_INSUFFICIENT_BUFFER = 0xF0000005; // 缓冲区空间不足
    public static uint DONGLE_NOT_INITIALIZED = 0xF0000006;	   // 产品尚未初始化 (即没设置PID)
    public static uint DONGLE_ALREADY_INITIALIZED = 0xF0000007; // 产品已经初始化 (即已设置PID)
    public static uint DONGLE_ADMINPIN_NOT_CHECK = 0xF0000008;  // 开发商密码没有验证
    public static uint DONGLE_USERPIN_NOT_CHECK = 0xF0000009;   // 用户密码没有验证
    public static uint DONGLE_INCORRECT_PIN = 0xF000FF00;	   // 密码不正确 (后2位指示剩余次数)
    public static uint DONGLE_PIN_BLOCKED = 0xF000000A;	       // PIN码已锁死
    public static uint DONGLE_ACCESS_DENIED = 0xF000000B;	   // 访问被拒绝 
    public static uint DONGLE_FILE_EXIST = 0xF000000E;		   // 文件已存在
    public static uint DONGLE_FILE_NOT_FOUND = 0xF000000F;      // 未找到指定的文件
    public static uint DONGLE_READ_ERROR = 0xF0000010;          // 读取数据错误
    public static uint DONGLE_WRITE_ERROR = 0xF0000011;         // 写入数据错误
    public static uint DONGLE_FILE_CREATE_ERROR = 0xF0000012;   // 创建文件错误
    public static uint DONGLE_FILE_READ_ERROR = 0xF0000013;     // 读取文件错误
    public static uint DONGLE_FILE_WRITE_ERROR = 0xF0000014;    // 写入文件错误
    public static uint DONGLE_FILE_DEL_ERROR = 0xF0000015;      // 删除文件错误
    public static uint DONGLE_FAILED = 0xF0000016;              // 操作失败
    public static uint DONGLE_CLOCK_EXPIRE = 0xF0000017;        // 加密锁时钟到期
    public static uint DONGLE_ERROR_UNKNOWN = 0xFFFFFFFF;	   // 未知的错误

    //网络锁相关操作
    public static uint DONGLE_FAILED_NET_CONNECTED = 0xF0000018;      //已经建立连接
    public static uint DONGLE_FAILED_NET_FULL = 0xF0000019;	         //连接数已满
    public static uint DONGLE_FAILED_NET_NOTFINDNETLOCK = 0xF0000020; //没有找到可用的网络加密锁
    public static uint DONGLE_FAILED_NET_NOCONNECTED = 0xF0000021;	 //还没有建立连接
    public static uint DONGLE_FAILED_NET_DELETED = 0xF0000022;		 //客户端已经断开
    public static uint DONGLE_FAILED_NET_DENIED = 0xF0000023;		 //客户被拒绝访问服务器

    //网络通讯返回值
    public static uint R_CONNECTFAILED = 0xC0000001;	        //连接失败
    public static uint R_SENDDATAFAILED = 0xC0000002;	    //发送数据失败
    public static uint R_SENDDATATIMEOUT = 0xC0000003;	    //发送数据超时
    public static uint R_CREATESOCKETFAILED = 0xC0000004;	//创建套接字失败
    public static uint R_RECVDATATIMEOUT = 0xC0000005;	    //接收数据超时
    public static uint R_RECVDATAERROR = 0xC0000006;	        //接收数据错误
}